//
//  lariat-online/include/LARASICFragment.h    (W.Badgett)
//
//  Description of the LARASIC raw data configuration format
//

#include <iostream>
#include <vector>

#ifndef _LARASICFragment_h
#define _LARASICFragment_h

class LARASICFragment
{
 public:
  enum
  {
    N_ASICS    = 30,  // Number of ASICs per LArIAT
    N_CHANNELS = 16,  // Number of channels (bytes) per ASIC
    N_GLOBAL   = 1,   // Number of global header bytes per ASIC
    N_SERIAL   = N_ASICS * ( N_CHANNELS + N_GLOBAL ),
    TOTAL_CHANNELS = N_ASICS * N_CHANNELS,
    SDF_M      = 0x1,
    SDF_S      = 0,
    SDC_M      = 0x1,
    SDC_S      = 1,
    ST_M       = 0x3,
    ST_S       = 2,
    SG_M       = 0x3,
    SG_S       = 4,
    SNC_M      = 0x1,
    SNC_S      = 6,
    STS_M      = 0x1,
    STS_S      = 7
  };

#pragma pack(push,1)

  typedef struct LarasicConfiguration_t
  {
    uint32_t fragmentSize;
    uint32_t fragmentType;
    uint32_t eventCounter;
    uint8_t  serialArray[N_SERIAL];
  } LarasicConfiguration_t;

#pragma pack(pop)

  LARASICFragment();

  static uint8_t gain(uint8_t serialByte)
  { return((serialByte >> SG_S ) & SG_M); }

  static uint8_t filter(uint8_t serialByte)
  { return((serialByte >> ST_S ) & ST_M); }

  static uint8_t dcOffset(uint8_t serialByte)
  { return((serialByte >> SNC_S ) & SNC_M); }

  static uint8_t testEnable(uint8_t serialByte)
  { return((serialByte >> STS_S ) & STS_M); }

  static std::vector<uint32_t> findTestChannels(LarasicConfiguration_t config);

  static uint8_t packChannel(uint8_t gain,
			     uint8_t filter,
			     uint8_t dcOffset,
			     uint8_t testEnable,
			     uint8_t outputCoupling,
			     uint8_t outputBuffer);
    
  // the unpacked object data

  LarasicConfiguration_t configuration;

  /**
     * stream the data from this object into a buffer.
     * Returns the number of characters written as a convenience.
     * This function sets fragmentSize and fragmentType.
     **/
  uint32_t streamToBuffer(std::ostream& buffer);
  
  /**
   * Fill in this object, assuming that the current position
   * in the buffer coincides with the start of this object.
   * Returns the number of characters read as a convenience.
   * Obviously streamToBuffer() and fillFromBuffer() must agree on
   * the structure of the object in the buffer!
   **/
  uint32_t fillFromBuffer(std::istream& buffer);

  
  /**
   * print this object in a user readable format
   **/
  void print(std::ostream& os = std::cout);
  static void printLarasicConfiguration(std::ostream& os, 
					LarasicConfiguration_t config);
};

std::ostream& operator<<(std::ostream& os, const LARASICFragment& e);
std::ostream& operator<<(std::ostream& os, 
			 const LARASICFragment::LarasicConfiguration_t& e);

#endif
